/*******************************************************************************
**	File:			testtx4.c
**	Author:			AES
**
**
**	Description:
**		Sample application to read from a CANpro port
**
**	Copyright (c) 2012, CTI, Connect Tech Inc. All Rights Reserved.
******************************************************************************/

#include <sys/socket.h>
#include <sys/select.h>
/* link this file with libsocket */
#include <gulliver.h>
#include <stdio.h>
#include <string.h>
#include <netdb.h>
#include <netinet/in.h>
#include <errno.h>
#include <alloca.h>
#include <devctl.h>
/* ...so flags to cc is "-lsocket -I../trunk" */

/* special address description flags for the CAN_ID */
#define CAN_EFF_FLAG 0x80000000U /* EFF/SFF is set in the MSB */
#define CAN_RTR_FLAG 0x40000000U /* remote transmission request */
#define CAN_ERR_FLAG 0x20000000U /* error frame */

/* valid bits in CAN ID for frame formats */
#define CAN_SFF_MASK 0x000007FFU /* standard frame format (SFF) */
#define CAN_EFF_MASK 0x1FFFFFFFU /* extended frame format (EFF) */
#define CAN_ERR_MASK 0x1FFFFFFFU /* omit EFF, RTR, ERR flags */

#define	AF_RAWCAN	AF_MAX + 1
/* don't exceed 255 */
#define	PF_RAWCAN	AF_RAWCAN

#pragma pack(1)
struct can_sockaddr {
/* compatiable to struct sockaddr defined in sys/socket.h */
	u_char			sa_len;			/* total length, 1 byte */
	sa_family_t		sa_family;		/* address family, 1 byte */
	uint16_t		iface;			/* interface, 2 byte. look at /dev/io-net/can* */
	uint32_t		id;				/* address value, 4-byte aligned */
	uint32_t		mask;			/* id & mask are big endian, MSB first*/
};
/*
 * CAN device statistics
 */
struct CanPerfStats {
	unsigned long ErrorWarning;
	unsigned long Overrun;
	unsigned long Wakeup;
	unsigned long BusError;
	unsigned long ErrorPassive;
	unsigned long ArbitrationLost;
	unsigned long Restarts;
	unsigned long BusErrorInit;
	unsigned long RxBytes;
	unsigned long RxPackets;
	unsigned long TxBytes;
	unsigned long TxPackets;
};
#pragma pack()

#define CTI_CMD_CODE      0x54A1  // see below
#define TIOGETSTAT      __DIOTF(_DCMD_MISC, CTI_CMD_CODE + 0, struct CanPerfStats)  /* Get port statistics */
#define TIOSETBAUD      __DIOTF(_DCMD_MISC, CTI_CMD_CODE + 1, unsigned long)  /* Set baud rate */

void print_stat(int fd)
{	
	int ret;
	struct CanPerfStats stats;
	
	ret = devctl(fd, TIOGETSTAT, &stats, sizeof(stats), NULL);
	if (ret != 0) {
		printf("TIOGETSTAT failed ret %x\n", ret);
		return;
	}
	
	printf("ErrorWarning %d\nOverrun %d\nWakeup %d\nBusError %d\nErrorPassive %d\nArbitrationLost %d\nRestarts %d\nBusErrorInit %d\nRxBytes %d\nRxPackets %d\nTxBytes %d\nTxPackets %d\n",
		stats.ErrorWarning,
		stats.Overrun,
		stats.Wakeup,
		stats.BusError,
		stats.ErrorPassive,
		stats.ArbitrationLost,
		stats.Restarts,
		stats.BusErrorInit,
		stats.RxBytes,
		stats.RxPackets,
		stats.TxBytes,
		stats.TxPackets);
}

int main( int argc, char *argv[] )
{
	int		s, i, packetcount = 0, ret;
	char	buffer[8];
	int		packet=0;
	struct can_sockaddr addr;
	fd_set	rfd;
	struct sockaddr *a;
	int		fromlen=0;	
	char	*tmp;
	unsigned long baud = 100; //100kb
	
	
	/* setup an address, using the first iface (0), and listen to every id */
	addr.sa_len = sizeof(addr);
	addr.sa_family = PF_RAWCAN;
	// pushing left to counter RTR bit		
	addr.mask = (0x123456 << 1) | 0x01;	// acceptance filter mask. Left shift to add RTR bit. 0="care", 1="don't care" 
	addr.id = (0x123456  << 1)  | 0x01;	// acceptance filter code. Left shift to add RTR bit.
	addr.iface = 0;		//not used

	a = (struct sockaddr *)alloca(sizeof(struct can_sockaddr));
	memset(a, 0, sizeof(struct can_sockaddr));

	// note to access individual ports you must add the port number to PF_RAWCAN
	// ie. port 2 is PF_RAWCAN+2
	s = socket(PF_RAWCAN+2, SOCK_RAW, 0);
	if(s==-1)
	{
		printf( "error opening socket, errno %d\n", errno );
		return 0;
	}
	
	// the following devctl shows how to set baud rate, setting of baud rate is an argument to driver start up though
	ret = devctl(s, TIOSETBAUD, &baud, sizeof(baud), NULL);
	if (ret != 0) {
		printf("TIOSETBAUD failed ret %x\n", ret);
		return;
	}

	// sets acceptance filter and mask
	ret = bind(s, (struct sockaddr *)&addr, sizeof(addr));
	if(ret < 0)
		return 0;
	
	FD_ZERO(&rfd);	

	while(1) {	
		fflush(stdout);
		FD_SET(s, &rfd);
		select(s+1, &rfd, 0, 0, 0);
		/* will be unblocked if anything received */
	
		fromlen = sizeof(struct can_sockaddr);
		ret = recvfrom(s, buffer, sizeof(buffer), 0, a, &fromlen);
		
		if(ret < 0) 
		{
			printf("Received error (%d - %s) from recvfrom()\n",	errno, strerror(errno));			
			break;
		}
		if(((struct can_sockaddr *)a)->id & CAN_RTR_FLAG)
			printf("ret %d Remote id:%x :Data:\n",ret,((struct can_sockaddr *)a)->id & ~CAN_RTR_FLAG);
		else
		{
			if(((struct can_sockaddr *)a)->id & CAN_EFF_FLAG)
				printf("ret %d Extended id:%x :Data:",ret,((struct can_sockaddr *)a)->id & ~CAN_EFF_FLAG);
			else
				printf("ret %d id:%x :Data:",ret,((struct can_sockaddr *)a)->id);
		}
		for(i=0;i<ret;i++)
				printf("%c",buffer[i]);
		packet++;
	}	
	close(s);

	return 0;
}
