/*******************************************************************************
**	File:			testtx4.c
**	Author:			AES
**
**
**	Description:
**		Sample application to write to a CANpro port
**
**	Copyright (c) 2012, CTI, Connect Tech Inc. All Rights Reserved.
******************************************************************************/

#include <sys/socket.h>
/* link this file with libsocket */
#include <gulliver.h>
#include <stdio.h>
#include <string.h>
#include <netdb.h>
#include <netinet/in.h>
#include <time.h>
#include <errno.h>
#include <devctl.h>

/* special address description flags for the CAN_ID */
#define CAN_EFF_FLAG 0x80000000U /* EFF/SFF is set in the MSB */
#define CAN_RTR_FLAG 0x40000000U /* remote transmission request */
#define CAN_ERR_FLAG 0x20000000U /* error frame */

/* valid bits in CAN ID for frame formats */
#define CAN_SFF_MASK 0x000007FFU /* standard frame format (SFF) */
#define CAN_EFF_MASK 0x1FFFFFFFU /* extended frame format (EFF) */
#define CAN_ERR_MASK 0x1FFFFFFFU /* omit EFF, RTR, ERR flags */


#define	AF_RAWCAN	AF_MAX + 1
/* don't exceed 255 */
#define	PF_RAWCAN	AF_RAWCAN

#pragma pack(1)
struct can_sockaddr {
	/* compatiable to struct sockaddr defined in sys/socket.h */
	u_char			sa_len;			/* total length, 1 byte */
	sa_family_t		sa_family;		/* address family, 1 byte */
	uint16_t		iface;			/* interface, 2 byte. look at /dev/io-net/can* */
	uint32_t		id;				/* address value, 4-byte aligned */
	uint32_t		mask;			/* id & mask are big endian, MSB first*/
};

/*
 * CAN device statistics
 */
struct CanPerfStats {
	unsigned long ErrorWarning;
	unsigned long Overrun;
	unsigned long Wakeup;
	unsigned long BusError;
	unsigned long ErrorPassive;
	unsigned long ArbitrationLost;
	unsigned long Restarts;
	unsigned long BusErrorInit;
	unsigned long RxBytes;
	unsigned long RxPackets;
	unsigned long TxBytes;
	unsigned long TxPackets;
};
#pragma pack()

#define CTI_CMD_CODE      0x54A1  // see below
#define TIOGETSTAT      __DIOTF(_DCMD_MISC, CTI_CMD_CODE + 0, struct CanPerfStats)  /* Get port statistics */
#define TIOSETBAUD      __DIOTF(_DCMD_MISC, CTI_CMD_CODE + 1, unsigned long)  /* Set baud rate */


void print_stat(int fd)
{	
	int ret;
	struct CanPerfStats stats;
	
	ret = devctl(fd, TIOGETSTAT, &stats, sizeof(stats), NULL);
	if (ret != 0) {
		printf("TIOGETSTAT failed ret %x\n", ret);
		return;
	}
	
	printf("ErrorWarning %d\nOverrun %d\nWakeup %d\nBusError %d\nErrorPassive %d\nArbitrationLost %d\nRestarts %d\nBusErrorInit %d\nRxBytes %d\nRxPackets %d\nTxBytes %d\nTxPackets %d\n",
		stats.ErrorWarning,
		stats.Overrun,
		stats.Wakeup,
		stats.BusError,
		stats.ErrorPassive,
		stats.ArbitrationLost,
		stats.Restarts,
		stats.BusErrorInit,
		stats.RxBytes,
		stats.RxPackets,
		stats.TxBytes,
		stats.TxPackets);
}

int main( int argc, char *argv[] )
{
	int		s, j, packetcount = 0, ret;
	struct	can_sockaddr addr;
	uint8_t buffer[100][8];
	time_t	start_time, end_time;
	int rtr = 0 ; // Remote Transmission Flag
	unsigned long baud = 100; //100kb

	addr.sa_len = sizeof(addr);
	addr.sa_family = PF_RAWCAN;
	addr.mask = 0xffffffff;		//not used
	addr.iface = 0;			//not used
	addr.id = 0x123456;

	if (addr.id > 0x7FF) {
		addr.id &= CAN_EFF_MASK;
		addr.id |= CAN_EFF_FLAG;
	} else
	 	addr.id &= CAN_SFF_MASK;
	if(rtr)
		addr.id |= CAN_RTR_FLAG;

	// note to access individual ports you must add the port number to PF_RAWCAN
	// ie. port 1 is PF_RAWCAN+1
	s = socket(PF_RAWCAN+1, SOCK_RAW, 0);
	if(s==-1)
	{
		printf( "error opening socket, errno = %d\n", errno );
		return 0;
	}
	// the following devctl shows how to set baud rate, setting of baud rate is an argument to driver start up though
	ret = devctl(s, TIOSETBAUD, &baud, sizeof(baud), NULL);
	if (ret != 0) {
		printf("TIOSETBAUD failed ret %x\n", ret);
		return;
	}
	
	start_time = time( NULL );
lloop:						
	if(packetcount > 8500)
	{
		// send 8600 packets as a test
		end_time = time( NULL );
		printf( "Elapsed time: %f seconds\n",
			difftime( end_time, start_time ) );
		printf("packetcount %d\n", packetcount );	
		print_stat(s);
		printf("ctrl-c to exit\n"); scanf("%d");
		return 0;		
	}
	for (j = 0; j < 100; j++)
	{			
		sprintf(buffer[j], "%5.5d\r\n\0",packetcount+j);		// send packet number in data field plus \cr\lf and NULL, handy for debugging
	}

	if(rtr)
		ret = sendto(s, NULL, NULL, 0, (struct sockaddr *)&addr, sizeof(addr));
	else
		ret = sendto(s, buffer, sizeof(buffer), 0, (struct sockaddr *)&addr, sizeof(addr));
	if (ret) {		
		packetcount +=ret/8;
	}
	if (ret < 0) return 0;		
	goto lloop;	

	close(s);

	return 0;
}
